module.exports = Controller;

function Controller (events, slideshowView) {
  addApiEventListeners(events, slideshowView);
  addNavigationEventListeners(events, slideshowView);
  addKeyboardEventListeners(events);
  addMouseEventListeners(events);
  addTouchEventListeners(events);
}

function addApiEventListeners(events, slideshowView) {
  events.on('pause', function(event) {
    removeKeyboardEventListeners(events);
    removeMouseEventListeners(events);
    removeTouchEventListeners(events);
  });

  events.on('resume',  function(event) {
    addKeyboardEventListeners(events);
    addMouseEventListeners(events);
    addTouchEventListeners(events);
  });
}

function addNavigationEventListeners (events, slideshowView) {
  if (slideshowView.isEmbedded()) {
    events.emit('gotoSlide', 1);
  }
  else {
    events.on('hashchange', navigateByHash);
    events.on('slideChanged', updateHash);

    navigateByHash();
  }

  events.on('message', navigateByMessage);

  function navigateByHash () {
    var slideNoOrName = (window.location.hash || '').substr(1);
    events.emit('gotoSlide', slideNoOrName);
  }

  function updateHash (slideNoOrName) {
    window.location.hash = '#' + slideNoOrName;
  }

  function navigateByMessage(message) {
    var cap;

    if ((cap = /^gotoSlide:(\d+)$/.exec(message.data)) !== null) {
      events.emit('gotoSlide', parseInt(cap[1], 10));
    }
  }
}

function removeKeyboardEventListeners(events) {
  events.removeAllListeners("keydown");
  events.removeAllListeners("keypress");
}

function addKeyboardEventListeners (events) {
  events.on('keydown', function (event) {
    switch (event.keyCode) {
      case 33: // Page up
      case 37: // Left
      case 38: // Up
        events.emit('gotoPreviousSlide');
        break;
      case 32: // Space
      case 34: // Page down
      case 39: // Right
      case 40: // Down
        events.emit('gotoNextSlide');
        break;
      case 36: // Home
        events.emit('gotoFirstSlide');
        break;
      case 35: // End
        events.emit('gotoLastSlide');
        break;
      case 27: // Escape
        events.emit('hideOverlay');
        break;
    }
  });

  events.on('keypress', function (event) {
    if (event.metaKey || event.ctrlKey) {
      // Bail out if meta or ctrl key was pressed
      return;
    }

    switch (String.fromCharCode(event.which)) {
      case 'j':
        events.emit('gotoNextSlide');
        break;
      case 'k':
        events.emit('gotoPreviousSlide');
        break;
      case 'c':
        events.emit('createClone');
        break;
      case 'p':
        events.emit('togglePresenterMode');
        break;
      case 'f':
        events.emit('toggleFullscreen');
        break;
      case '?':
        events.emit('toggleHelp');
        break;
    }
  });
}

function removeMouseEventListeners(events) {
  events.removeAllListeners("mousewheel");
}

function addMouseEventListeners (events) {
  events.on('mousewheel', function (event) {
    if (event.wheelDeltaY > 0) {
      events.emit('gotoPreviousSlide');
    }
    else if (event.wheelDeltaY < 0) {
      events.emit('gotoNextSlide');
    }
  });
}


function removeTouchEventListeners(events) {
  events.removeAllListeners("touchstart");
  events.removeAllListeners("touchend");
  events.removeAllListeners("touchmove");
}


function addTouchEventListeners (events) {
  var touch
    , startX
    , endX
    ;

  var isTap = function () {
    return Math.abs(startX - endX) < 10;
  };

  var handleTap = function () {
    events.emit('tap', endX);
  };

  var handleSwipe = function () {
    if (startX > endX) {
      events.emit('gotoNextSlide');
    }
    else {
      events.emit('gotoPreviousSlide');
    }
  };

  events.on('touchstart', function (event) {
    touch = event.touches[0];
    startX = touch.clientX;
  });

  events.on('touchend', function (event) {
    if (event.target.nodeName.toUpperCase() === 'A') {
      return;
    }

    touch = event.changedTouches[0];
    endX = touch.clientX;

    if (isTap()) {
      handleTap();
    }
    else {
      handleSwipe();
    }
  });

  events.on('touchmove', function (event) {
    event.preventDefault();
  });
}
